#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "cdbslib.h"
#include "error.h"
#include "config.h"
#include "uniqname.h"

static void	find_last (char *, char *, char *, char *);
static char* 	find_suffix (char * fname, int);

/* THRUNAME -- Calculate a new unique name for a throughput file
 *
 * i: ref		reference file descriptor
 * i: reftype		reference file type
 * i: root_suffix       suffix that gets appended to the rootname
 * i: cdbsdir		the directory the reference file will be stored in
 * o: newroot		the unique root name
 * i: maxch		maximum length of calculated name
 *
 * History:
 * 06/12/02  45993  mss   add THERMAL type
 */

void thruname (reffile *ref, char *reftype, char *root_suffix, char *cdbsdir, 
	       char *newroot, int maxch)
{
    char	cdbsname[SZ_FNAME+1], cdbsroot[SZ_FNAME+1];
    char        wildcard[SZ_FNAME+1];
    char	*suffix;
    int		count;

    /* Get the root prefix from information 
     * in the reference file header 
     */

    getsynroot (ref, reftype, cdbsroot, SZ_FNAME);

    /* Get the last filename with the root prefix
     * and use it to compute the root suffix
     */

    strcpy(wildcard, "_???_");
    strcat(wildcard, root_suffix);
    strcat(wildcard, ".fits");

    find_last (cdbsdir, cdbsroot, wildcard, cdbsname);

    if (cdbsname[0] == EOS) {
      /* If name not found with new suffix, search for it with old suffix */

        find_last (cdbsdir, cdbsroot, "_???.tab", cdbsname);

	/* Extract version number from name */

	if (cdbsname[0] == EOS) { 
	    count = 0;
	} else {
	    suffix = find_suffix (cdbsname, 1);
	    count = atoi (suffix);
	}

    } else {
	/* Extract version number from name */

        suffix = find_suffix (cdbsname, 2);
	count = atoi (suffix);
    }

    /* Compute the new root name */

    sprintf (newroot, "%s_%03d_%s", cdbsroot, count+1, root_suffix);
}

/* Find the most recent file with a specified name and suffix
 *
 * i: dir		directory files are stored in
 * i: rootname		rootname of files to search
 * i: suffix		suffix containing pattern to match
 * o: fname		most recent filename matching pattern
 */

static void find_last (char *dir, char *rootname, char *suffix, char *fname) 
{
    char	pattern[SZ_FNAME+1];

    /* Construct pattern */

    safecopy (pattern, dir, SZ_FNAME);
    safecat (pattern, rootname, SZ_FNAME);
    safecat (pattern, suffix, SZ_FNAME);

    /* Search for file matching pattern */

    lastfile (pattern, fname, SZ_FNAME);

    /* Set match to null string if not found */

    if (strcmp (pattern, fname) == 0)
        fname[0] = EOS;
}

/* FIND_SUFFIX -- Return the starting position of the suffix
 *
 * i: fname		file name containing suffix
 * i: ucount		number of underscores in suffix
 */

static char *find_suffix (char *fname, int ucount) 
{
    int		ic, count;

    count = 0;
    for (ic = strlen (fname) - 1; ic >= 0; ic --) {
        if (fname[ic] == '_')
	    if (++count == ucount)
	        break;
    }

    return (fname + ic + 1);
}
